# -*- coding: utf_8 -*-
"""MobSF File Upload and Home Routes."""
import json
import logging
import os
import platform
import re
import shutil
from pathlib import Path
from datetime import timedelta
from wsgiref.util import FileWrapper

from django.conf import settings
from django.utils.timezone import now
from django.core.paginator import Paginator
from django.http import HttpResponse, HttpResponseRedirect
from django.views.decorators.http import require_http_methods
from django.utils import timezone
from django.shortcuts import (
    redirect,
    render,
)
from django.template.defaulttags import register

from mobsf.MobSF.forms import FormUtil, UploadFileForm
from mobsf.MobSF.utils import (
    MD5_REGEX,
    get_md5,
    is_dir_exists,
    is_file_exists,
    is_md5,
    key,
    print_n_send_error_response,
    python_dict,
)
from mobsf.MobSF.init import api_key
from mobsf.MobSF.security import is_safe_path, sanitize_filename, sanitize_svg
from mobsf.MobSF.views.helpers import FileType
from mobsf.MobSF.views.scanning import Scanning
from mobsf.MobSF.views.apk_downloader import apk_download
from mobsf.StaticAnalyzer.models import (
    EnqueuedTask,
    RecentScansDB,
    StaticAnalyzerAndroid,
    StaticAnalyzerIOS,
    StaticAnalyzerWindows,
)
from mobsf.DynamicAnalyzer.views.common.shared import (
    invalid_params,
    send_response,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    MAINTAINER_GROUP,
    Permissions,
    permission_required,
)

LINUX_PLATFORM = ['Darwin', 'Linux']
HTTP_BAD_REQUEST = 400
HTTP_STATUS_404 = 404
HTTP_SERVER_ERROR = 500
logger = logging.getLogger(__name__)
register.filter('key', key)


@login_required
def index(request):
    """Index Route."""
    mimes = (settings.APK_MIME
             + settings.IPA_MIME
             + settings.ZIP_MIME
             + settings.APPX_MIME)
    exts = (settings.ANDROID_EXTS
            + settings.IOS_EXTS
            + settings.WINDOWS_EXTS)
    context = {
        'version': settings.MOBSF_VER,
        'mimes': mimes,
        'exts': '|'.join(exts),
    }
    template = 'general/home.html'
    return render(request, template, context)


class Upload(object):
    """Handle File Upload based on App type."""

    def __init__(self, request):
        self.request = request
        self.form = UploadFileForm(request.POST, request.FILES)
        self.file_type = None
        self.file = None

    @staticmethod
    @login_required
    @permission_required(Permissions.SCAN)
    def as_view(request):
        upload = Upload(request)
        return upload.upload_html()

    def resp_json(self, data):
        resp = HttpResponse(json.dumps(data),
                            content_type='application/json; charset=utf-8')
        resp['Access-Control-Allow-Origin'] = '*'
        return resp

    def upload_html(self):
        request = self.request
        response_data = {
            'description': '',
            'status': 'error',
        }
        if request.method != 'POST':
            msg = 'Method not Supported!'
            logger.error(msg)
            response_data['description'] = msg
            return self.resp_json(response_data)

        if not self.form.is_valid():
            msg = 'Invalid Form Data!'
            logger.error(msg)
            response_data['description'] = msg
            return self.resp_json(response_data)

        self.file = request.FILES['file']
        self.file_type = FileType(self.file)
        if not self.file_type.is_allow_file():
            msg = 'File format not Supported!'
            logger.error(msg)
            response_data['description'] = msg
            return self.resp_json(response_data)

        if self.file_type.is_ipa():
            if platform.system() not in LINUX_PLATFORM:
                msg = 'Static Analysis of iOS IPA requires Mac or Linux'
                logger.error(msg)
                response_data['description'] = msg
                return self.resp_json(response_data)

        response_data = self.upload()
        return self.resp_json(response_data)

    def upload_api(self):
        """API File Upload."""
        api_response = {}
        request = self.request
        if not self.form.is_valid():
            api_response['error'] = FormUtil.errors_message(self.form)
            return api_response, HTTP_BAD_REQUEST
        self.file = request.FILES['file']
        self.file_type = FileType(self.file)
        if not self.file_type.is_allow_file():
            api_response['error'] = 'File format not Supported!'
            return api_response, HTTP_BAD_REQUEST
        api_response = self.upload()
        return api_response, 200

    def upload(self):
        request = self.request
        scanning = Scanning(request)
        content_type = self.file.content_type
        file_name = sanitize_filename(self.file.name)
        logger.info('MIME Type: %s FILE: %s', content_type, file_name)
        if self.file_type.is_apk():
            return scanning.scan_apk()
        elif self.file_type.is_xapk():
            return scanning.scan_xapk()
        elif self.file_type.is_apks():
            return scanning.scan_apks()
        elif self.file_type.is_aab():
            return scanning.scan_aab()
        elif self.file_type.is_jar():
            return scanning.scan_jar()
        elif self.file_type.is_aar():
            return scanning.scan_aar()
        elif self.file_type.is_so():
            return scanning.scan_so()
        elif self.file_type.is_zip():
            return scanning.scan_zip()
        elif self.file_type.is_ipa():
            return scanning.scan_ipa()
        elif self.file_type.is_dylib():
            return scanning.scan_dylib()
        elif self.file_type.is_a():
            return scanning.scan_a()
        elif self.file_type.is_appx():
            return scanning.scan_appx()


@login_required
def api_docs(request):
    """Api Docs Route."""
    key = '*******'
    try:
        if (settings.DISABLE_AUTHENTICATION == '1'
                or request.user.is_staff
                or request.user.groups.filter(name=MAINTAINER_GROUP).exists()):
            key = api_key(settings.MOBSF_HOME)
    except Exception:
        logger.exception('[ERROR] Failed to get API key')
    context = {
        'title': 'API Docs',
        'api_key': key,
        'version': settings.MOBSF_VER,
    }
    template = 'general/apidocs.html'
    return render(request, template, context)


def about(request):
    """About Route."""
    context = {
        'title': 'About',
        'version': settings.MOBSF_VER,
    }
    template = 'general/about.html'
    return render(request, template, context)


def donate(request):
    """Donate Route."""
    context = {
        'title': 'Donate',
        'version': settings.MOBSF_VER,
    }
    template = 'general/donate.html'
    return render(request, template, context)


def error(request):
    """Error Route."""
    context = {
        'title': 'Error',
        'version': settings.MOBSF_VER,
    }
    template = 'general/error.html'
    return render(request, template, context)


def zip_format(request):
    """Zip Format Message Route."""
    context = {
        'title': 'Zipped Source Instruction',
        'version': settings.MOBSF_VER,
    }
    template = 'general/zip.html'
    return render(request, template, context)


def robots_txt(request):
    content = 'User-agent: *\nDisallow: /*/\nAllow: /*\n'
    return HttpResponse(content, content_type='text/plain')


@login_required
def dynamic_analysis(request):
    """Dynamic Analysis Landing."""
    context = {
        'title': 'Dynamic Analysis',
        'version': settings.MOBSF_VER,
    }
    template = 'general/dynamic.html'
    return render(request, template, context)


@login_required
def recent_scans(request, page_size=10, page_number=1):
    """Show Recent Scans Route."""
    entries = []
    paginator = Paginator(
        RecentScansDB.objects.all().order_by('-TIMESTAMP').values(), page_size)
    page_obj = paginator.get_page(page_number)
    page_obj.page_size = page_size
    md5_list = [i['MD5'] for i in page_obj]

    android = StaticAnalyzerAndroid.objects.filter(
        MD5__in=md5_list).only(
            'PACKAGE_NAME', 'VERSION_NAME', 'FILE_NAME', 'MD5')
    ios = StaticAnalyzerIOS.objects.filter(
        MD5__in=md5_list).only('FILE_NAME', 'MD5')

    updir = Path(settings.UPLD_DIR)
    icon_mapping = {}
    package_mapping = {}
    for item in android:
        package_mapping[item.MD5] = item.PACKAGE_NAME
        icon_mapping[item.MD5] = item.ICON_PATH
    for item in ios:
        icon_mapping[item.MD5] = item.ICON_PATH

    for entry in page_obj:
        if entry['MD5'] in package_mapping.keys():
            entry['PACKAGE'] = package_mapping[entry['MD5']]
        else:
            entry['PACKAGE'] = ''
        entry['ICON_PATH'] = icon_mapping.get(entry['MD5'], '')

        if entry['FILE_NAME'].endswith('.ipa'):
            entry['BUNDLE_HASH'] = get_md5(
                entry['PACKAGE_NAME'].encode('utf-8'))
            report_file = updir / entry['BUNDLE_HASH'] / 'mobsf_dump_file.txt'
        else:
            report_file = updir / entry['MD5'] / 'logcat.txt'
        entry['DYNAMIC_REPORT_EXISTS'] = report_file.exists()
        entries.append(entry)
    context = {
        'title': 'Recent Scans',
        'entries': entries,
        'version': settings.MOBSF_VER,
        'page_obj': page_obj,
        'async_scans': settings.ASYNC_ANALYSIS,
    }
    template = 'general/recent.html'
    return render(request, template, context)


@login_required
@permission_required(Permissions.SCAN)
def download_apk(request):
    """Download and APK by package name."""
    package = request.POST['package']
    # Package validated in apk_download()
    context = {
        'status': 'failed',
        'description': 'Unable to download APK',
    }
    res = apk_download(package)
    if res:
        context = res
        context['status'] = 'ok'
        context['package'] = package
    resp = HttpResponse(
        json.dumps(context),
        content_type='application/json; charset=utf-8')
    return resp


@login_required
def search(request, api=False):
    """Search scan by checksum or text."""
    if request.method == 'POST':
        query = request.POST['query']
    else:
        query = request.GET['query']

    if not query:
        msg = 'No search query provided.'
        return print_n_send_error_response(request, msg, api)

    checksum = query if re.match(MD5_REGEX, query) else find_checksum(query)

    if checksum and re.match(MD5_REGEX, checksum):
        db_obj = RecentScansDB.objects.filter(MD5=checksum).first()
        if db_obj:
            url = f'/{db_obj.ANALYZER}/{db_obj.MD5}/'
            if api:
                return {'checksum': db_obj.MD5}
            else:
                return HttpResponseRedirect(url)

    msg = 'You can search by MD5, app name, package name, or file name.'
    return print_n_send_error_response(request, msg, api, 'Scan not found')


def find_checksum(query):
    """Get the first matching checksum from the database."""
    search_fields = ['FILE_NAME', 'PACKAGE_NAME', 'APP_NAME']

    for field in search_fields:
        result = RecentScansDB.objects.filter(
            **{f'{field}__icontains': query}).first()
        if result:
            return result.MD5

    return None

# AJAX


@login_required
@require_http_methods(['POST'])
def scan_status(request, api=False):
    """Get Current Status of a scan in progress."""
    try:
        scan_hash = request.POST['hash']
        if not is_md5(scan_hash):
            return invalid_params(api)
        robj = RecentScansDB.objects.filter(MD5=scan_hash)
        if not robj.exists():
            data = {'status': 'failed', 'error': 'scan hash not found'}
            return send_response(data, api)
        data = {'status': 'ok', 'logs': python_dict(robj[0].SCAN_LOGS)}
    except Exception as exp:
        logger.exception('Fetching Scan Status')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)


def file_download(dwd_file, filename, content_type):
    """HTTP file download response."""
    def create_response(content, is_binary=True):
        """Helper function to create HTTP response."""
        if is_binary:
            wrapper = FileWrapper(content)
            response = HttpResponse(wrapper, content_type=content_type)
            response['Content-Length'] = dwd_file.stat().st_size
        else:
            response = HttpResponse(content, content_type=content_type)
        if filename:
            # Remove CRLF from filename to prevent header injection
            safe_filename = filename.replace('\r', '').replace('\n', '')
            val = f'attachment; filename="{safe_filename}"'
            response['Content-Disposition'] = val
        return response

    # Handle SVG files with bleach cleaning to prevent XSS attacks
    if dwd_file.suffix == '.svg':
        with open(dwd_file, 'r', encoding='utf-8') as file:
            svg_content = file.read()
            cleaned_svg = sanitize_svg(svg_content)
            return create_response(cleaned_svg, is_binary=False)

    # Handle all other binary file types
    with open(dwd_file, 'rb') as file:
        return create_response(file)


@login_required
@require_http_methods(['GET'])
def download_binary(request, checksum, api=False):
    """Download binary from uploads directory."""
    try:
        allowed_exts = settings.ALLOWED_EXTENSIONS
        if not is_md5(checksum):
            return HttpResponse(
                'Invalid MD5 Hash',
                status=HTTP_STATUS_404)
        robj = RecentScansDB.objects.filter(MD5=checksum).first()
        if not robj:
            return HttpResponse(
                'Scan hash not found',
                status=HTTP_STATUS_404)
        file_ext = f'.{robj.SCAN_TYPE}'
        if file_ext not in allowed_exts.keys():
            return HttpResponse(
                'Invalid Scan Type',
                status=HTTP_STATUS_404)
        filename = f'{checksum}{file_ext}'
        dwd_file = Path(settings.UPLD_DIR) / checksum / filename
        if not dwd_file.exists():
            return HttpResponse(
                'File not found',
                status=HTTP_STATUS_404)
        return file_download(
            dwd_file,
            sanitize_filename(robj.FILE_NAME),
            allowed_exts[file_ext])
    except Exception:
        logger.exception('Download Binary Failed')
        return HttpResponse(
            'Failed to download file due to an error',
            status=HTTP_SERVER_ERROR)


@login_required
@require_http_methods(['GET'])
def download(request):
    """Download from mobsf downloads directory."""
    root = settings.DWD_DIR
    filename = request.path.replace('/download/', '', 1)
    dwd_file = Path(root) / filename

    # Security Checks
    if not is_safe_path(root, dwd_file, filename):
        msg = 'Path Traversal Attack Detected'
        return print_n_send_error_response(request, msg)

    # File and Extension Check
    ext = dwd_file.suffix
    allowed_exts = settings.ALLOWED_EXTENSIONS
    if ext in allowed_exts and dwd_file.is_file():
        return file_download(
            dwd_file,
            None,
            allowed_exts[ext])

    # Special Case for Certain Image Files
    if filename.endswith(('screen/screen.png', '-icon.png')):
        return HttpResponse('')

    return HttpResponse(status=HTTP_STATUS_404)


@login_required
def generate_download(request):
    """Generate downloads for smali/java zip."""
    try:
        logger.info('Generating Downloads')
        md5 = request.GET['hash']
        file_type = request.GET['file_type']
        if (not is_md5(md5)
                or file_type not in ('smali', 'java')):
            msg = 'Invalid download type or hash'
            logger.exception(msg)
            return print_n_send_error_response(request, msg)
        app_dir = Path(settings.UPLD_DIR) / md5
        dwd_dir = Path(settings.DWD_DIR)
        file_name = ''
        if file_type == 'java':
            # For Java zipped source code
            directory = app_dir / 'java_source'
            dwd_file = dwd_dir / f'{md5}-java'
            shutil.make_archive(
                dwd_file.as_posix(), 'zip', directory.as_posix())
            file_name = f'{md5}-java.zip'
        elif file_type == 'smali':
            # For Smali zipped source code
            directory = app_dir / 'smali_source'
            dwd_file = dwd_dir / f'{md5}-smali'
            shutil.make_archive(
                dwd_file.as_posix(), 'zip', directory.as_posix())
            file_name = f'{md5}-smali.zip'
        return redirect(f'/download/{file_name}')
    except Exception:
        msg = 'Generating Downloads'
        logger.exception(msg)
        return print_n_send_error_response(request, msg)


@login_required
@permission_required(Permissions.DELETE)
@require_http_methods(['POST'])
def delete_scan(request, api=False):
    """Delete Scan from DB and remove the scan related files."""
    try:
        if api:
            md5_hash = request.POST['hash']
        else:
            md5_hash = request.POST['md5']

        if not re.match(MD5_REGEX, md5_hash):
            return send_response({'deleted': 'Invalid scan hash'}, api)

        # Delete DB Entries
        scan = RecentScansDB.objects.filter(MD5=md5_hash)
        if not scan.exists():
            return send_response({'deleted': 'Scan not found in Database'}, api)
        if settings.ASYNC_ANALYSIS:
            # Handle Async Tasks
            et = EnqueuedTask.objects.filter(checksum=md5_hash).first()
            if et:
                max_time_passed = now() - et.created_at > timedelta(
                    minutes=settings.ASYNC_ANALYSIS_TIMEOUT)
                if not (et.completed_at or max_time_passed):
                    # Queue is in progress, cannot delete the task
                    return send_response(
                        {'deleted': 'A scan can only be deleted after it is completed'},
                        api)
        # Delete all related DB entries
        EnqueuedTask.objects.filter(checksum=md5_hash).all().delete()
        RecentScansDB.objects.filter(MD5=md5_hash).delete()
        StaticAnalyzerAndroid.objects.filter(MD5=md5_hash).delete()
        StaticAnalyzerIOS.objects.filter(MD5=md5_hash).delete()
        StaticAnalyzerWindows.objects.filter(MD5=md5_hash).delete()
        # Delete Upload Dir Contents
        app_upload_dir = os.path.join(settings.UPLD_DIR, md5_hash)
        if is_dir_exists(app_upload_dir):
            shutil.rmtree(app_upload_dir)
        # Delete Download Dir Contents
        dw_dir = settings.DWD_DIR
        for item in os.listdir(dw_dir):
            item_path = os.path.join(dw_dir, item)
            valid_item = item.startswith(md5_hash + '-')
            # Delete all related files
            if is_file_exists(item_path) and valid_item:
                os.remove(item_path)
            # Delete related directories
            if is_dir_exists(item_path) and valid_item:
                shutil.rmtree(item_path, ignore_errors=True)
        return send_response({'deleted': 'yes'}, api)
    except Exception as exp:
        msg = str(exp)
        exp_doc = exp.__doc__
        return print_n_send_error_response(request, msg, api, exp_doc)


class RecentScans(object):

    def __init__(self, request):
        self.request = request

    def recent_scans(self):
        page = self.request.GET.get('page', 1)
        page_size = self.request.GET.get('page_size', 10)
        result = RecentScansDB.objects.all().values().order_by('-TIMESTAMP')
        try:
            paginator = Paginator(result, page_size)
            content = paginator.page(page)
            data = {
                'content': list(content),
                'count': paginator.count,
                'num_pages': paginator.num_pages,
            }
        except Exception as exp:
            data = {'error': str(exp)}
        return data


def update_scan_timestamp(scan_hash):
    # Update the last scan time.
    tms = timezone.now()
    RecentScansDB.objects.filter(MD5=scan_hash).update(TIMESTAMP=tms)



# -*- coding: utf_8 -*-
"""Module for strings-method for java."""
import logging
from pathlib import Path

from mobsf.StaticAnalyzer.views.common.shared_func import (
    is_secret_key,
    strings_and_entropies,
    url_n_email_extract,
)
from mobsf.StaticAnalyzer.views.common.entropy import (
    get_entropies,
)
from mobsf.MobSF.utils import (
    GOOGLE_API_KEY_REGEX,
    GOOGLE_APP_ID_REGEX,
    append_scan_status,
    get_android_src_dir,
)

logger = logging.getLogger(__name__)


def strings_from_so(checksum, elf_strings):
    """Extract Strings from so file."""
    msg = 'Extracting String data from SO'
    logger.info(msg)
    append_scan_status(checksum, msg)
    sos = []
    try:
        for solib in elf_strings:
            for so, str_list in solib.items():
                if not str_list:
                    continue
                # extract url, email
                so_str = ' '.join(str_list)
                so_urls, so_urls_nf, so_emails_nf = url_n_email_extract(
                    so_str, so)
                sos.append({so: {
                    'secrets': list(get_entropies(so_str)),
                    'strings': list(set(str_list)),
                    'urls_list': so_urls,
                    'urls_nf': so_urls_nf,
                    'emails_nf': so_emails_nf,
                }})
    except Exception as exp:
        msg = 'Failed to extract String data from SO'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return sos


def strings_from_apk(checksum, app_dic):
    """Extract Strings from an APK."""
    results = {
        'strings': [],
        'urls_list': [],
        'urls_nf': [],
        'emails_nf': [],
        'secrets': [],
        'firebase_creds': {},
    }
    try:
        msg = 'Extracting String data from APK'
        logger.info(msg)
        append_scan_status(checksum, msg)

        rsrc = app_dic.get('androguard_apk_resources')
        if rsrc:
            pkg = rsrc.get_packages_names()[0]
            rsrc.get_strings_resources()

            # Iterate over resource strings
            for _res_key, res_value in rsrc.values.get(pkg, {}).items():
                res_string = res_value.get('string')
                if not res_string:
                    continue

                for key, value in res_string:
                    if not value:
                        continue

                    # Extract Firebase credentials
                    if key == 'google_api_key' and GOOGLE_API_KEY_REGEX.match(value):
                        results['firebase_creds']['google_api_key'] = value
                    elif key == 'google_app_id' and GOOGLE_APP_ID_REGEX.match(value):
                        results['firebase_creds']['google_app_id'] = value

                    # Format and collect strings
                    formatted_str = f'"{key}" : "{value}"'
                    results['strings'].append(formatted_str)

                    # Check for possible secrets
                    if is_secret_key(key) and ' ' not in value:
                        results['secrets'].append(formatted_str)
        elif app_dic.get('apk_strings'):
            # No secret key check for APK strings
            results['strings'] = app_dic['apk_strings']
        else:
            msg = 'Failed to extract String data from APK'
            logger.warning(msg)
            append_scan_status(checksum, msg)
            return results

        # Extract URLs and Emails from collected strings
        results['strings'] = list(set(results['strings']))
        ul, u_nf, e_nf = url_n_email_extract(
            ''.join(results['strings']), 'Android String Resource')
        results['urls_list'], results['urls_nf'], results['emails_nf'] = ul, u_nf, e_nf

    except Exception as exp:
        msg = 'Failed to extract String data from APK'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))

    return results


def strings_from_code(checksum, src_dir, typ, exts):
    """Extract Strings and Secrets from Java/Kotlin code."""
    msg = 'Extracting String data from Code'
    logger.info(msg)
    append_scan_status(checksum, msg)
    data = {
        'strings': set(),
        'secrets': set(),
    }
    try:
        src_dir = get_android_src_dir(Path(src_dir), typ)
        data = strings_and_entropies(checksum, src_dir, exts)
    except Exception as exp:
        msg = 'Failed to extract String data from Code'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return data


def get_strings_metadata(app_dic, elf_strings, exts, code_dic):
    """Get Strings, secrets, entropies, URLs, emails."""
    checksum = app_dic['md5']
    typ = app_dic['zipped']
    app_dir = app_dic['app_dir']
    strings = {
        'strings_apk_res': {},
        'strings_so': [],
        'strings_code': {},
    }
    urls_list = []
    urls_n_files = []
    emails_n_files = []
    secrets = []
    if app_dic.get('androguard_string_resources') or app_dic.get('apk_strings'):
        # APK
        apk_res = strings_from_apk(checksum, app_dic)
        code_dic['firebase_creds'] = apk_res['firebase_creds']
        strings['strings_apk_res'] = apk_res['strings']
        urls_list.extend(apk_res['urls_list'])
        urls_n_files.extend(apk_res['urls_nf'])
        emails_n_files.extend(apk_res['emails_nf'])
        secrets.extend(apk_res['secrets'])
    if elf_strings:
        # ELF (.so) by file
        sos = strings_from_so(checksum, elf_strings)
        so_strings = []
        for so in sos:
            for so_file, s in so.items():
                so_strings.append({so_file: s['strings']})
                urls_list.extend(s['urls_list'])
                urls_n_files.extend(s['urls_nf'])
                emails_n_files.extend(s['emails_nf'])
                secrets.extend(s['secrets'])
        secrets = list(set(secrets))
        strings['strings_so'] = so_strings

    if exts:
        # Source Code
        code_res = strings_from_code(checksum, app_dir, typ, exts)
        strings['strings_code'] = list(code_res['strings'])
        secrets.extend(code_res['secrets'])

    code_dic['strings'] = strings
    code_dic['secrets'] = list(secrets)
    # Code Analysis has urls, urlsnfiles and emailsnfiles
    code_dic['urls'].extend(urls_n_files)
    code_dic['emails'].extend(emails_n_files)
    code_dic['urls_list'].extend(urls_list)
    code_dic['urls_list'] = list(set(code_dic['urls_list']))



# -*- coding: utf_8 -*-
"""Xposed tests."""
import logging
import base64
import json
import os
import re

from mobsf.DynamicAnalyzer.views.android.environment import (
    Environment,
)
from mobsf.MobSF.utils import (
    is_base64,
    is_file_exists,
    python_list,
)

logger = logging.getLogger(__name__)


def get_hooked_apis():
    """Hooked APIs Regex."""
    apis = {
        'api_fileio': {
            'name': 'File IO',
            'regex': r'libcore.io|android.app.Shared'
                    r'PreferencesImpl\\$EditorImpl',
            'icon': 'fas fa-file-signature',
        },
        'api_reflect': {
            'name': 'Reflection',
            'regex': r'java.lang.reflect',
            'icon': 'fas fa-object-ungroup',
        },
        'api_sysprop': {
            'name': 'Device Data',
            'regex': r'android.content.ContentResolver|'
                     r'android.location.Location|'
                     r'android.media.AudioRecord|'
                     r'android.media.MediaRecorder|'
                     r'android.os.SystemProperties',
            'icon': 'fas fa-phone',
        },
        'api_binder': {
            'name': 'Binder',
            'regex': r'android.app.Activity|'
                     r'android.app.ContextImpl|'
                     r'android.app.ActivityThread',
            'icon': 'fas fa-cubes',
        },
        'api_crypto': {
            'name': 'Crypto',
            'regex': r'javax.crypto.spec.SecretKeySpec|'
                     r'javax.crypto.Cipher|'
                     r'javax.crypto.Mac',
            'icon': 'fas fa-lock',
        },
        'api_acntmnger': {
            'name': 'System Managers',
            'regex': r'android.accounts.AccountManager|'
                     r'android.app.ApplicationPackageManager|'
                     r'android.app.NotificationManager|'
                     r'android.net.ConnectivityManager|'
                     r'android.content.BroadcastReceiver',
            'icon': 'fas fa-cogs',
        },
        'api_deviceinfo': {
            'name': 'Device Info',
            'regex': r'android.telephony.TelephonyManager|'
                     r'android.net.wifi.WifiInfo|'
                     r'android.os.Debug',
            'icon': 'fas fa-info',
        },
        'api_dexloader': {
            'name': 'Dex Class Loader',
            'regex': r'dalvik.system.BaseDexClassLoader|'
                     r'dalvik.system.DexFile|'
                     r'dalvik.system.DexClassLoader|'
                     r'dalvik.system.PathClassLoader',
            'icon': 'fas fa-asterisk',
        },
        'api_cmd': {
            'name': 'Process',
            'regex': r'java.lang.Runtime|java.lang.ProcessBuilder|'
                     r'java.io.FileOutputStream|'
                     r'java.io.FileInputStream|'
                     r'android.os.Process',
            'icon': 'fas fa-chart-bar',
        },
        'api_cntvl': {
            'name': 'Content Values',
            'regex': r'android.content.ContentValues',
            'icon': 'fas fa-bars',
        },
        'api_sms': {
            'name': 'SMS Manager',
            'regex': r'android.telephony.SmsManager',
            'icon': 'fas fa-comment-alt',
        },
        'api_net': {
            'name': 'Network',
            'regex': r'java.net.URL|org.apache.http.'
                     r'impl.client.AbstractHttpClient',
            'icon': 'fas fa-wifi',
        },
        'api_base64': {
            'name': 'Base64',
            'regex': r'android.util.Base64',
            'icon': 'fas fa-puzzle-piece',
        },
    }
    return apis


def droidmon_api_analysis(app_dir, package):
    """API Analysis."""
    try:
        dat = ''
        hooked_apis = get_hooked_apis()
        api_details = {}
        hooks = []
        location = os.path.join(app_dir, 'x_logcat.txt')
        if not is_file_exists(location):
            return {}
        logger.info('Xposed Droidmon API Analysis')
        with open(location, 'r',
                  encoding='utf8',
                  errors='ignore') as flip:
            dat = flip.readlines()
        res_id = 'Droidmon-apimonitor-' + package + ':'
        for line in dat:
            if res_id not in line:
                continue
            _, value = line.split(res_id, 1)
            try:
                apis = json.loads(value, strict=False)
                call_data = {}
                call_data['class'] = apis['class']
                call_data['method'] = apis['method']
                if apis.get('return'):
                    call_data['return'] = apis['return']
                if apis.get('args'):
                    call_data['args'] = apis['args']
                for api, details in hooked_apis.items():
                    if re.findall(details['regex'], apis['class']):
                        call_data['api'] = api
                        # Decode Base64
                        if ('decode' in apis['method']
                                and api == 'api_base64'):
                            call_data['decoded'] = base64_decode(
                                call_data['args']).decode('utf-8', 'ignore')
                        hooks.append(call_data)
            except Exception:
                pass
        for hook in hooks:
            iden = hook['api']
            api_details[iden] = {
                'name': hooked_apis[iden]['name'],
                'icon': hooked_apis[iden]['icon'],
                'calls': [],
            }
        for hook in hooks:
            iden = hook['api']
            if hook not in api_details[iden]['calls']:
                api_details[iden]['calls'].append(hook)
    except Exception:
        logger.exception('Droidmon API Analysis')
    return api_details


def base64_decode(args):
    """Decode Base64 Automatically."""
    decoded = ''
    args_list = python_list(args)
    if not is_base64(args_list[0]):
        return decoded
    try:
        decoded = base64.b64decode(
            args_list[0]).decode('ISO-8859-1')
    except Exception:
        pass
    return decoded


def download_xposed_log(apk_dir):
    """Download Xposed Output."""
    env = Environment()
    xposed_out = ('/data/data/'
                  'de.robv.android.xposed.installer'
                  '/log/error.log')
    env.adb_command(['pull',
                     xposed_out,
                     apk_dir + 'x_logcat.txt'])
    logger.info('Downloading droidmon API monitor logs')



"""SAML2 SSO logic."""
from urllib.parse import urlparse
import logging

from onelogin.saml2.auth import (
    OneLogin_Saml2_Auth,
)
from onelogin.saml2.idp_metadata_parser import (
    OneLogin_Saml2_IdPMetadataParser,
)

from django.conf import settings
from django.contrib.auth.models import (
    Group,
    User,
)
from django.contrib.auth import login
from django.urls import reverse
from django.shortcuts import redirect
from django.views.decorators.http import require_http_methods

from mobsf.MobSF.views.authorization import (
    MAINTAINER_GROUP,
    VIEWER_GROUP,
)
from mobsf.MobSF.utils import (
    print_n_send_error_response,
)
from mobsf.MobSF.security import (
    sanitize_redirect,
)

logger = logging.getLogger(__name__)
ASSERTION_IDS = set()


def get_url_components(url):
    """Get URL components."""
    purl = urlparse(url)
    return purl.scheme, purl.netloc, purl.port


def init_saml_auth(req):
    """Initialize SAML auth."""
    host = req['sp_url']
    acs_route = reverse('saml_acs')
    saml_settings = {
        'strict': True,
        'debug': True,
        'sp': {
            'entityId': f'{host}{acs_route}',
            'assertionConsumerService': {
                'url': f'{host}{acs_route}',
                'binding': ('urn:oasis:names:tc:'
                            'SAML:2.0:bindings:HTTP-POST'),
            },
        },
        'idp': {
            'entityId': settings.IDP_ENTITY_ID,
            'singleSignOnService': {
                'url': settings.IDP_SSO_URL,
                'binding': ('urn:oasis:names:tc:'
                            'SAML:2.0:bindings:HTTP-Redirect'),
            },
            'x509cert': settings.IDP_X509CERT,
        },
        'security': {
            'requestedAuthnContext': False,
        },
    }
    try:
        idp_data = None
        if settings.IDP_METADATA_URL:
            idp_data = OneLogin_Saml2_IdPMetadataParser.parse_remote(
                settings.IDP_METADATA_URL,
                timeout=5)
        if idp_data:
            saml_settings['idp'] = idp_data['idp']
    except Exception:
        logger.exception('[ERROR] parsing IdP metadata URL.')
    return OneLogin_Saml2_Auth(req, saml_settings)


def prepare_django_request(request):
    """Prepare Django request for SAML."""
    scheme = 'https' if request.is_secure() else 'http'
    netloc = request.get_host()
    port = request.get_port()
    if settings.SP_HOST:
        scheme, netloc, port = get_url_components(
            settings.SP_HOST.strip('/'))
        if not port:
            port = 443 if scheme == 'https' else 80
    https_state = 'on' if scheme == 'https' else 'off'
    sp_url = f'{scheme}://{netloc}'
    result = {
        'https': https_state,
        'http_host': netloc,
        'server_port': port,
        'script_name': request.get_full_path_info(),
        'get_data': request.GET.copy(),
        'post_data': request.POST.copy(),
        'lowercase_urlencoding': bool(settings.IDP_IS_ADFS == '1'),
        'query_string': request.META['QUERY_STRING'],
        'sp_url': sp_url,
    }
    return result


def check_replay(auth):
    """Check for replay attack."""
    request_id = auth.get_last_assertion_id()
    if request_id:
        if request_id in ASSERTION_IDS:
            raise Exception('Replay attack detected.')
        ASSERTION_IDS.add(request_id)


def get_redirect_url(req):
    """Check for open redirect and return redirect url."""
    redirect_url = '/'
    if 'RelayState' not in req['post_data']:
        return redirect_url
    relay_state = req['post_data']['RelayState']
    # Allow only relative URLs
    if relay_state:
        redirect_url = sanitize_redirect(relay_state)
    return redirect_url


def get_user_role(roles):
    """Get user role."""
    mrole = any(MAINTAINER_GROUP.lower() in gp.lower() for gp in roles)
    if mrole:
        return MAINTAINER_GROUP
    return VIEWER_GROUP


@require_http_methods(['GET'])
def saml_login(request):
    """Handle SSO Login."""
    try:
        if settings.DISABLE_AUTHENTICATION == '1':
            return redirect('/')
        req = prepare_django_request(request)
        auth = init_saml_auth(req)
        nextp = request.GET.get('next', '')
        redirect_url = sanitize_redirect(nextp)
        return redirect(auth.login(return_to=redirect_url))
    except Exception as exp:
        return print_n_send_error_response(
            request,
            exp,
            False)


@require_http_methods(['POST'])
def saml_acs(request):
    """Handle SSO Assertion Consumer Service."""
    try:
        if settings.DISABLE_AUTHENTICATION == '1':
            return redirect('/')
        req = prepare_django_request(request)
        auth = init_saml_auth(req)
        auth.process_response()
        check_replay(auth)
        if not auth.is_authenticated():
            raise Exception(
                'SAML authentication failed.')
        # Extract user attributes for AuthZ and AuthN
        attributes = auth.get_attributes()
        if not attributes.get('email'):
            raise Exception(
                'email attribute not found in SAML response.')
        if not attributes.get('role'):
            raise Exception(
                'role attribute not found in SAML response.')
        email = attributes['email'][0]
        role = get_user_role(attributes['role'])
        if User.objects.filter(username=email).exists():
            user = User.objects.get(username=email)
            user.groups.clear()
            user.groups.add(Group.objects.get(name=role))
            login(request, user)
        else:
            user = User.objects.create_user(
                username=email,
                email=email)
            user.is_staff = False
            user.groups.add(Group.objects.get(name=role))
            login(request, user)
        return redirect(get_redirect_url(req))
    except Exception as exp:
        return print_n_send_error_response(
            request,
            exp,
            False)



from django import forms
from django.contrib.auth.models import User
from django.contrib.auth.forms import UserCreationForm


class UploadFileForm(forms.Form):
    file = forms.FileField()


class FormUtil(object):

    def __init__(self, form):
        self.form = form

    @staticmethod
    def errors_message(form):
        """Form Errors.

        :param form forms.Form
        form.errors.get_json_data() django 2.0 or higher

        :return
        example
        {
        "error": {
            "file": "This field is required.",
            "test": "This field is required."
            }
        }
        """
        data = form.errors.get_json_data()
        for k, v in data.items():
            data[k] = ' '.join([value_detail['message'] for value_detail in v])
        return data

    @staticmethod
    def errors(form):
        return form.errors.get_json_data()


class RegisterForm(UserCreationForm):

    role = forms.ChoiceField(
        choices=(('viewer', 'Viewer'), ('maintainer', 'Maintainer')),
        required=True,
        help_text='User Role')

    def clean_email(self):
        email = self.cleaned_data.get('email')
        if User.objects.filter(email__iexact=email).exists():
            raise forms.ValidationError('Email already exists')
        return email

    class Meta:
        """Meta Class."""

        model = User
        fields = ['username', 'password1', 'password2', 'email', 'role']



# -*- coding: utf_8 -*-
"""iOS Dynamic Analysis."""
import logging
import os
import time
from threading import Event, Thread
from pathlib import Path

from django.conf import settings
from django.shortcuts import render
from django.views.decorators.http import require_http_methods


from mobsf.MobSF.utils import (
    IOS_DEVICE_ID_REGEX,
    SSH_DEVICE_ID_REGEX,
    get_md5,
    is_md5,
    parse_host_port,
    print_n_send_error_response,
    strict_package_check,
)
from mobsf.DynamicAnalyzer.views.common.shared import (
    invalid_params,
    send_response,
)
from mobsf.MobSF.security import is_attack_pattern
from mobsf.DynamicAnalyzer.forms import UploadFileForm
from mobsf.DynamicAnalyzer.views.ios.helpers import (
    configure_proxy,
    get_local_ipa_list,
)
from mobsf.DynamicAnalyzer.views.ios.device.device import IOSDevice
from mobsf.DynamicAnalyzer.views.ios.device.connect import IOSConnector
from mobsf.DynamicAnalyzer.views.ios.device.environment import IOSEnvironment
from mobsf.DynamicAnalyzer.views.ios.device.frida import FridaIOSDevice
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)

logger = logging.getLogger(__name__)
OK = 'ok'
FAILED = 'failed'


@login_required
@permission_required(Permissions.SCAN)
def dynamic_analysis_device(request, api=False):
    """The iOS Device Dynamic Analysis Entry point."""
    try:
        if api:
            device_id = request.POST.get('device_id', '')
        else:
            device_id = request.GET.get('device_id', '')
        scan_apps = get_local_ipa_list()
        connector = IOSConnector()
        wifi_devices = get_ios_devices_over_wifi()
        if os.getenv('MOBSF_PLATFORM') == 'docker':
            devices = []
        else:
            devices = connector.get_usb_devices()
        context = {
            'selected_device': device_id,
            'apps': scan_apps,
            'usb_devices': devices,
            'wifi_devices': wifi_devices,
            'title': 'MobSF Dynamic Analysis Jailbroken Device',
            'version': settings.MOBSF_VER,
        }
        if api:
            return context
        template = 'dynamic_analysis/ios/device/dynamic_analysis.html'
        return render(request, template, context)
    except Exception as exp:
        logger.exception('iOS Jailbroken Device Dynamic Analysis')
        return print_n_send_error_response(request, exp, api)


@login_required
@permission_required(Permissions.SCAN)
def dynamic_analyzer_device(request, api=False):
    """Dynamic Analyzer for Jailbroken iOS devices."""
    try:
        if api:
            bundleid = request.POST.get('bundle_id')
            device_id = request.POST.get('device_id')
            form = None
        else:
            bundleid = request.GET.get('bundle_id')
            device_id = request.GET.get('device_id')
            form = UploadFileForm()
        if not bundleid or not strict_package_check(bundleid):
            return print_n_send_error_response(
                request,
                'Invalid iOS Bundle id',
                api)
        checksum = get_md5(bundleid.encode('utf-8'))
        ios_device, error = validate_and_connect_device(device_id)
        if error:
            return print_n_send_error_response(
                request,
                error,
                api)

        app_dir = Path(settings.UPLD_DIR) / checksum
        if not app_dir.exists():
            app_dir.mkdir()
        configure_proxy(request, bundleid, None)
        env = IOSEnvironment(ios_device)
        env.setup_or_start_frida()
        context = {
            'checksum': checksum,
            'device_id': device_id,
            'bundle_id': bundleid,
            'version': settings.MOBSF_VER,
            'form': form,
            'title': 'iOS Device Dynamic Analyzer'}
        template = 'dynamic_analysis/ios/device/dynamic_analyzer.html'
        if api:
            return context
        return render(request, template, context)
    except Exception:
        logger.exception('iOS Device Dynamic Analyzer')
        return print_n_send_error_response(
            request,
            'iOS Device Dynamic Analysis Failed.',
            api)


# JSON API
@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def ios_instrument_device(request, api=False):
    """Instrument app with frida."""
    data = {
        'status': FAILED,
        'message': ''}
    try:
        action = request.POST.get('frida_action', 'spawn')
        pid = request.POST.get('pid')
        new_bundle_id = request.POST.get('new_bundle_id')
        device_id = request.POST['device_id']
        bundle_id = request.POST['bundle_id']
        md5_hash = get_md5(bundle_id.encode('utf-8'))
        default_hooks = request.POST['default_hooks']
        dump_hooks = request.POST['dump_hooks']
        auxiliary_hooks = request.POST['auxiliary_hooks']
        code = request.POST['frida_code']

        ios_device, error = validate_and_connect_device(device_id)
        if error:
            data['message'] = error
            return send_response(data, api)
        if not strict_package_check(bundle_id):
            data['message'] = 'Invalid iOS Bundle id'
            return send_response(data, api)
        if new_bundle_id and not strict_package_check(new_bundle_id):
            data['message'] = 'Invalid iOS Bundle id'
            return send_response(data, api)

        # Fill extras
        extras = {}
        screenshot_action = request.POST.get('screenshot_action')
        if screenshot_action:
            extras['screenshot_action'] = screenshot_action.strip()
        class_name = request.POST.get('class_name')
        if class_name:
            extras['class_name'] = class_name.strip()
        class_search = request.POST.get('class_search')
        if class_search:
            extras['class_search'] = class_search.strip()
        method_search = request.POST.get('method_search')
        if method_search:
            extras['method_search'] = method_search.strip()
        cls_trace = request.POST.get('class_trace')
        if cls_trace:
            extras['class_trace'] = cls_trace.strip()
        if (is_attack_pattern(default_hooks)
                or is_attack_pattern(dump_hooks)
                or is_attack_pattern(auxiliary_hooks)
                or not is_md5(md5_hash)):
            return invalid_params(api)
        frida_obj = FridaIOSDevice(
            ios_device,
            None,
            md5_hash,
            bundle_id,
            default_hooks.split(','),
            dump_hooks.split(','),
            auxiliary_hooks.split(','),
            extras,
            code)
        if action == 'run':
            frida_obj.run_app()
        if action == 'spawn':
            logger.info('Starting Instrumentation')
            frida_obj.spawn()
        elif action == 'ps':
            logger.info('Enumerating running applications')
            data['message'] = frida_obj.ps()
        elif action == 'get':
            # Get injected Frida script.
            data['message'] = frida_obj.get_script(nolog=True)
        elif action == 'capture':
            # Capture screenshot
            screenshot_event = Event()
            frida_obj.screenshot_event = screenshot_event
            Thread(
                target=download_screenshot,
                args=(ios_device, md5_hash, screenshot_event),
                daemon=True).start()

        if action in ('spawn', 'session', 'capture'):
            if pid and pid.isdigit():
                # Attach to a different pid/bundle id
                args = (int(pid), new_bundle_id)
                logger.info('Attaching to %s [PID: %s]', new_bundle_id, pid)
            else:
                # Injecting to existing session/spawn
                if action == 'session':
                    logger.info('Injecting to existing frida session')
                args = (None, None)
            Thread(target=frida_obj.session, args=args, daemon=True).start()
        data['status'] = OK
    except Exception as exp:
        logger.exception('Frida Instrumentation failed')
        data['message'] = str(exp)
    return send_response(data, api)


# JSON API
@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def ssh_execute_device(request, api=False):
    """Execute commands in iOS device over SSH."""
    data = {
        'status': FAILED,
        'message': 'Failed to execute command'}
    try:
        device_id = request.POST['device_id']
        cmd = request.POST['cmd']
        ios_device, error = validate_and_connect_device(device_id)
        if error:
            data['message'] = error
            return send_response(data, api)
        out, err, _ = ios_device.execute_command(cmd)
        message = f'{out}\n{err}'
        data['status'] = OK
        data['message'] = message
    except Exception as exp:
        data['message'] = str(exp)
        logger.exception('Executing Commands in iOS device')
    return send_response(data, api)


# JSON API
@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def upload_file_device(request, api=False):
    """Upload file to device."""
    err_msg = 'Failed to upload file'
    data = {
        'status': FAILED,
        'message': err_msg,
    }
    try:
        device_id = request.POST['device_id']
        ios_device, error = validate_and_connect_device(device_id)
        if error:
            data['message'] = error
            return send_response(data, api)
        form = UploadFileForm(request.POST, request.FILES)
        if form.is_valid():
            fobject = request.FILES['file']
            upload = ios_device.upload_file_object(
                fobject, fobject.name)
            if upload:
                data['status'] = OK
                data['message'] = 'Successfully uploaded file'
            else:
                data['message'] = 'Failed to upload file'
    except Exception as exp:
        logger.exception(err_msg)
        data['message'] = str(exp)
    return send_response(data, api)


# JSON API
@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST', 'GET'])
def system_logs_device(request, api=False):
    """Show system logs."""
    data = {
        'status': FAILED,
        'message': 'Failed to get system logs',
    }
    try:
        pid = request.GET.get('pid')
        if request.method == 'POST':
            device_id = request.POST['device_id']
            if pid and not pid.isdigit():
                data['message'] = 'Invalid PID'
                return send_response(data, api)
            ios_device, error = validate_and_connect_device(device_id)
            if error:
                data['message'] = error
                return send_response(data, api)
            cmd = 'oslog'
            if pid:
                cmd = f'oslog -p {pid}'
            out, err, _ = ios_device.execute_command(cmd, timeout=9)
            message = f'{out}\n{err}'
            data['status'] = OK
            data['message'] = message
            return send_response(data, api)
        # GET request
        logger.info('Getting logs')
        device_id = request.GET['device_id']
        if pid and not pid.isdigit():
            data['message'] = 'Invalid PID'
            return send_response(data, api)
        usb = IOS_DEVICE_ID_REGEX.match(device_id)
        wifi = SSH_DEVICE_ID_REGEX.match(device_id)
        if not (usb or wifi):
            data['message'] = 'Invalid device id or SSH connection string format'
            return send_response(data, api)
        template = 'dynamic_analysis/ios/device/system_logs.html'
        return render(request,
                      template,
                      {'device_id': device_id,
                       'pid': pid,
                       'version': settings.MOBSF_VER,
                       'title': 'Live logs'})
    except Exception as exp:
        logger.exception('Getting logs')
        data['message'] = str(exp)
    return send_response(data, api)


# JSON API
@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def ps_device(request, api=False):
    """Get list of running processes on iOS device."""
    data = {
        'status': FAILED,
        'message': 'Failed to get process list',
    }
    try:
        device_id = request.POST.get('device_id', '')
        ios_device, error = validate_and_connect_device(device_id)
        if error:
            data['message'] = error
            return send_response(data, api)
        processes = ios_device.ps()
        if processes:
            data['status'] = OK
            data['message'] = processes
        else:
            data['message'] = 'No processes found'

    except Exception as exp:
        logger.exception('Getting process list failed')
        data['message'] = str(exp)
    return send_response(data, api)


# JSON API
@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def get_ios_device(request, api=False):
    """Get details of selected iOS device."""
    data = {
        'status': 'failed',
        'message': 'Failed to connect to iOS device',
    }
    try:
        device_id = request.POST.get('device_id', '')
        ios_device, error = validate_and_connect_device(device_id)
        if error:
            data['message'] = error
            return send_response(data, api)
        data['apps'] = ios_device.list_applications()
        data['status'] = OK
        data['message'] = 'Successfully connected to iOS device'
    except Exception:
        logger.exception('Failed to connect to iOS device')
    return send_response(data, api)


# JSON API
@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def install_ipa_device(request, api=False):
    """Install an IPA file on the device."""
    data = {
        'status': 'failed',
        'message': 'Failed to install IPA on device',
    }
    try:
        device_id = request.POST.get('device_id', '')
        # Checksum from IPA SAST Upload location
        checksum = request.POST.get('checksum', '')
        if not is_md5(checksum):
            data['message'] = 'Invalid checksum format'
            return send_response(data, api)
        ios_device, error = validate_and_connect_device(device_id)
        if error:
            data['message'] = error
            return send_response(data, api)
        if not ios_device.install_ipa(checksum):
            data['message'] = 'Failed to install IPA'
            return send_response(data, api)
        data['status'] = OK
        data['message'] = 'Successfully installed IPA'
    except Exception:
        logger.exception('Failed to install IPA on iOS device')
    return send_response(data, api)


# Helpers
def get_ios_devices_over_wifi():
    """Get available iOS devices over WiFi."""
    devices = []
    connect_strings = None
    connect_strings_settings = getattr(settings, 'IOS_ANALYZER_IDENTIFIERS', '')
    if os.getenv('MOBSF_IOS_ANALYZER_IDENTIFIERS'):
        connect_strings = os.getenv('MOBSF_IOS_ANALYZER_IDENTIFIERS')
    elif connect_strings_settings:
        connect_strings = connect_strings_settings
    try:
        if not connect_strings:
            return devices
        if ',' in connect_strings:
            # Multiple devices
            for device in connect_strings.split(','):
                devices.append(parse_host_port(device.strip()))
        else:
            # Single device
            devices.append(parse_host_port(connect_strings.strip()))
    except ValueError:
        pass
    return devices


def get_ios_device_wifi_connect_string(ssh_string):
    """Get iOS device connect string."""
    available_devices = get_ios_devices_over_wifi()
    if not ssh_string:
        return None, None
    ip, port = parse_host_port(ssh_string)
    if (ip, port) in available_devices:
        return ip, port
    return None, None


def download_screenshot(ios_device, checksum, screenshot_event=None):
    """Download screenshot from device."""
    try:
        # Wait for the Frida script to signal screenshot completion
        # rather than using a fixed sleep, with a 30s timeout as fallback.
        if screenshot_event:
            screenshot_event.wait(timeout=30)
        else:
            time.sleep(5)
        timestamp = time.strftime('%Y%m%d%H%M%S')
        path = Path(settings.DWD_DIR) / f'{checksum}-sshot-{timestamp}.png'
        ios_device.download_file('/tmp/screenshot.png', path)
    except Exception as exp:
        logger.error('Failed to download screenshot from iOS device')
        logger.error(exp)


def validate_and_connect_device(device_id):
    """Validate device_id and establish connection to iOS device."""
    usb = IOS_DEVICE_ID_REGEX.match(device_id)
    wifi = SSH_DEVICE_ID_REGEX.match(device_id)
    if not (usb or wifi):
        return None, 'Invalid device id or SSH connection string format'

    ssh_user = getattr(settings, 'IOS_SSH_USER', 'root')
    ssh_password = getattr(settings, 'IOS_SSH_PASSWORD', 'alpine')

    connector = IOSConnector()
    if usb:
        connected = connector.connect_usb(
            device_id, username=ssh_user, password=ssh_password)
        if not connected:
            return None, 'Failed to connect to iOS device over USB'
    else:
        ip, port = get_ios_device_wifi_connect_string(device_id)
        connected = connector.connect_wifi(
            ip, ssh_user, ssh_password, port=port)
        if not connected:
            return None, 'Failed to connect to iOS device over WiFi'
    ios_device = IOSDevice(connector)
    return ios_device, None



"""Handle (.so) Shared Object file."""
import logging

from django.conf import settings
from django.shortcuts import render

import mobsf.MalwareAnalyzer.views.Trackers as Trackers
import mobsf.MalwareAnalyzer.views.VirusTotal as VirusTotal
from mobsf.MobSF.utils import (
    append_scan_status,
    file_size,
    print_n_send_error_response,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    get_symbols,
    hash_gen,
)
from mobsf.StaticAnalyzer.views.common.firebase import (
    firebase_analysis,
)
from mobsf.StaticAnalyzer.views.common.binary.lib_analysis import (
    library_analysis,
)
from mobsf.StaticAnalyzer.views.android.strings import (
    get_strings_metadata,
)
from mobsf.StaticAnalyzer.models import (
    StaticAnalyzerAndroid,
)
from mobsf.StaticAnalyzer.views.android.db_interaction import (
    get_context_from_db_entry,
    save_get_ctx,
)
from mobsf.MalwareAnalyzer.views.MalwareDomainCheck import MalwareDomainCheck
from mobsf.MobSF.views.authorization import (
    Permissions,
    has_permission,
)

logger = logging.getLogger(__name__)


def so_analysis(request, app_dic, rescan, api):
    checksum = app_dic['md5']
    app_dic['app_file'] = f'{app_dic["md5"]}.so'  # NEW FILENAME
    app_dic['app_path'] = (app_dic['app_dir'] / app_dic['app_file']).as_posix()
    app_dic['app_dir'] = app_dic['app_dir'].as_posix() + '/'
    db_entry = StaticAnalyzerAndroid.objects.filter(MD5=checksum)
    if db_entry.exists() and not rescan:
        context = get_context_from_db_entry(db_entry)
    else:
        if not has_permission(request, Permissions.SCAN, api):
            return print_n_send_error_response(
                request,
                'Permission Denied',
                False)
        append_scan_status(checksum, 'init')
        # Analysis starts here
        app_dic['size'] = f'{str(file_size(app_dic["app_path"]))}MB'
        app_dic['sha1'], app_dic['sha256'] = hash_gen(
            checksum,
            app_dic['app_path'])
        app_dic['zipped'] = 'so'
        app_dic['files'] = []
        app_dic['file_analysis'] = []
        app_dic['playstore'] = {'error': True}
        app_dic['manifest_file'] = None
        app_dic['manifest_namespace'] = None
        app_dic['manifest_parsed_xml'] = None
        app_dic['mani'] = ''
        man_data_dic = {
            'services': [],
            'activities': [],
            'receivers': [],
            'providers': [],
            'libraries': [],
            'categories': [],
            'perm': {},
            'packagename': app_dic['app_name'],
            'mainactivity': '',
            'min_sdk': '',
            'max_sdk': '',
            'target_sdk': '',
            'androver': '',
            'androvername': '',
            'icons': [],
        }
        man_an_dic = {
            'manifest_anal': [],
            'exported_act': [],
            'exported_cnt': {
                'exported_activities': 0,
                'exported_services': 0,
                'exported_receivers': 0,
                'exported_providers': 0,
            },
            'browsable_activities': {},
            'permissions': {},
            'network_security': {
                'network_findings': [],
                'network_summary': {},
            },
            'malware_permissions': {},
        }
        cert_dic = {
            'certificate_info': '',
            'certificate_findings': [],
            'certificate_summary': {},
        }
        app_dic['real_name'] = ''
        elf_dict = library_analysis(
            checksum,
            app_dic['app_dir'],
            'elf')
        # File Analysis is used to store symbols from so
        app_dic['file_analysis'] = get_symbols(
            elf_dict['elf_symbols'])
        apkid_results = {}
        code_an_dic = {
            'api': {},
            'behaviour': {},
            'perm_mappings': {},
            'findings': {},
            'niap': {},
            'urls_list': [],
            'urls': [],
            'emails': [],
            'sbom': {},
        }
        # Get the strings and metadata from shared object
        get_strings_metadata(
            app_dic,
            elf_dict['elf_strings'],
            None,
            code_an_dic)
        # Firebase DB Check
        code_an_dic['firebase'] = firebase_analysis(
            checksum,
            code_an_dic)
        # Domain Extraction and Malware Check
        code_an_dic['domains'] = MalwareDomainCheck().scan(
            checksum,
            code_an_dic['urls_list'])
        # Extract Trackers from Domains
        trk = Trackers.Trackers(
            checksum,
            None,
            app_dic['tools_dir'])
        trackers = trk.get_trackers_domains_or_deps(
            code_an_dic['domains'], [])
        context = save_get_ctx(
            app_dic,
            man_data_dic,
            man_an_dic,
            code_an_dic,
            cert_dic,
            elf_dict['elf_analysis'],
            apkid_results,
            trackers,
            rescan,
        )
    context['appsec'] = {}
    context['average_cvss'] = None
    context['dynamic_analysis_done'] = False
    context['virus_total'] = None
    if settings.VT_ENABLED:
        vt = VirusTotal.VirusTotal(checksum)
        context['virus_total'] = vt.get_result(
            app_dic['app_path'])
    template = 'static_analysis/android_binary_analysis.html'
    if api:
        return context
    else:
        return render(request, template, context)



def check_transport_security(p_list):
    """Check info.plist for insecure connection configurations."""
    ats = []
    if 'NSAppTransportSecurity' in p_list:
        ats_dict = p_list['NSAppTransportSecurity']
        if ats_dict and ats_dict.get('NSAllowsArbitraryLoads'):
            ats.append({
                'issue': ('App Transport Security '
                          'AllowsArbitraryLoads is allowed'),
                'severity': 'high',
                'description': (
                    'App Transport Security restrictions are disabled '
                    'for all network connections. Disabling ATS means that '
                    'unsecured HTTP connections are allowed. HTTPS '
                    'connections are also allowed, and are still subject '
                    'to default server trust evaluation. However, '
                    'extended security checks like requiring a minimum '
                    'Transport Layer Security (TLS) protocol version—are '
                    'disabled. This setting is not applicable to domains '
                    'listed in NSExceptionDomains.'),
            })
        if ats_dict and ats_dict.get('NSAllowsArbitraryLoadsForMedia'):
            ats.append({
                'issue': 'Insecure media load is allowed',
                'severity': 'high',
                'description': (
                    'App Transport Security restrictions are disabled for '
                    'media loaded using the AVFoundation framework, '
                    'without affecting your URLSession connections. '
                    'This setting is not applicable to domains '
                    'listed in NSExceptionDomains.'),
            })
        if ats_dict and ats_dict.get('NSAllowsArbitraryLoadsInWebContent'):
            ats.append({
                'issue': 'Insecure WebView load is allowed',
                'severity': 'high',
                'description': (
                    'App Transport Security restrictions are disabled for '
                    'requests made from WebViews without affecting your '
                    'URLSession connections. This setting is not applicable '
                    'to domains listed in NSExceptionDomains.'),
            })
        if ats_dict and ats_dict.get('NSAllowsLocalNetworking'):
            ats.append({
                'issue': 'Insecure local networking is allowed',
                'severity': 'high',
                'description': (
                    'App Transport Security restrictions are disabled for '
                    'requests made from local networking '
                    'without affecting your '
                    'URLSession connections. This setting is not applicable '
                    'to domains listed in NSExceptionDomains.'),
            })

        # NS Domain Exceptions
        if ats_dict and ats_dict.get('NSExceptionDomains'):
            exception_domains = ats_dict.get('NSExceptionDomains')
            ats.append({
                'issue': 'NSExceptionDomains',
                'severity': 'info',
                'description': ', '.join(exception_domains.keys()),
            })
            for domain, config in exception_domains.items():
                if not isinstance(config, dict):
                    continue
                old_exp = 'NSTemporaryExceptionAllowsInsecureHTTPLoads'
                old_exp2 = 'NSThirdPartyExceptionAllowsInsecureHTTPLoads'
                if (config.get('NSExceptionAllowsInsecureHTTPLoads', False)
                        or config.get(old_exp, False)
                        or config.get(old_exp2, False)):
                    if domain in {'localhost', '127.0.0.1'}:
                        continue
                    findings = {
                        'issue': ('Insecure communication'
                                  ' to {} is allowed'.format(domain)),
                        'severity': 'high',
                        'description': (
                            'NSExceptionAllowsInsecureHTTPLoads allows '
                            'insecure HTTP loads to {}, '
                            'or to be able to loosen the '
                            'server trust evaluation '
                            'requirements for HTTPS '
                            'connections to the domain.'.format(domain)
                        ),
                    }
                    ats.append(findings)

                if config.get('NSIncludesSubdomains', False):
                    findings = {
                        'issue': ('NSIncludesSubdomains set to TRUE'
                                  ' for {}'.format(domain)),
                        'severity': 'info',
                        'description': (
                            'NSIncludesSubdomains applies the ATS exceptions '
                            'for the given domain to all '
                            'subdomains as well. '
                            'For example, the ATS exceptions in the '
                            'domain exception dictionary apply to {}, '
                            'as well as math.{}, history.{}, and so on. '
                            'Otherwise, if the value is NO, the exceptions '
                            'apply only to '
                            '{}.'.format(domain, domain, domain, domain)
                        ),
                    }
                    ats.append(findings)
                old_tls = 'NSTemporaryExceptionMinimumTLSVersion'
                inc_min_tls = (config.get('NSExceptionMinimumTLSVersion', None)
                               or config.get(old_tls, None))
                if inc_min_tls in ['TLSv1.0', 'TLSv1.1']:
                    findings = {
                        'issue': ('NSExceptionMinimumTLSVersion set to {}'
                                  ' on {}'.format(inc_min_tls, domain)),
                        'severity': 'high',
                        'description': (
                            'The minimum Transport Layer '
                            'Security (TLS) version '
                            'for network connections sent to {} '
                            'is set to {}. This version is deemed '
                            'to be insecure'.format(domain, inc_min_tls)
                        ),
                    }
                    ats.append(findings)

                elif inc_min_tls == 'TLSv1.2':
                    findings = {
                        'issue': ('NSExceptionMinimumTLSVersion set to {}'
                                  ' on {}'.format(inc_min_tls, domain)),
                        'severity': 'warning',
                        'description': (
                            'The minimum Transport Layer '
                            'Security (TLS) version '
                            'for network connections sent to {} '
                            'is set to {}. '
                            'This version is vulnerable to '
                            'attacks such as POODLE, FREAK, '
                            'or CurveSwap etc.'.format(domain, inc_min_tls)
                        ),
                    }
                    ats.append(findings)

                elif inc_min_tls == 'TLSv1.3':
                    findings = {
                        'issue': ('NSExceptionMinimumTLSVersion set to {}'
                                  ' on {}'.format(inc_min_tls, domain)),
                        'severity': 'secure',
                        'description': (
                            'The minimum Transport Layer '
                            'Security (TLS) version '
                            'for network connections sent to {} '
                            'is set to {}.'.format(domain, inc_min_tls)
                        ),
                    }
                    ats.append(findings)

                elif inc_min_tls is None:
                    pass

                else:
                    findings = {
                        'issue': ('NSExceptionMinimumTLSVersion set to {}'
                                  ' on {}'.format(inc_min_tls, domain)),
                        'severity': 'info',
                        'description': (
                            'The minimum Transport Layer '
                            'Security (TLS) version '
                            'for network connections sent to {} '
                            'is set to {}.'.format(domain, inc_min_tls)
                        ),
                    }
                    ats.append(findings)
                old1 = config.get(
                    'NSTemporaryExceptionRequiresForwardSecrecy', False)
                old2 = config.get(
                    'NSThirdPartyExceptionRequiresForwardSecrecy', False)
                cur = config.get(
                    'NSExceptionRequiresForwardSecrecy', False)
                fsc = [old1, old2, cur]
                if (old1 or old2 or cur) and 'NO' in fsc:
                    findings = {
                        'issue': ('NSExceptionRequiresForwardSecrecy '
                                  'set to NO'
                                  ' for {}'.format(domain)),
                        'severity': 'high',
                        'description': (
                            'NSExceptionRequiresForwardSecrecy '
                            'limits the accepted ciphers to '
                            'those that support perfect '
                            'forward secrecy (PFS) through the '
                            'Elliptic Curve Diffie-Hellman '
                            'Ephemeral (ECDHE) key exchange. '
                            'Set the value for this key to NO to override '
                            'the requirement that a server must support '
                            'PFS for the given domain. This key is optional. '
                            'The default value is YES, which limits the '
                            'accepted ciphers to those that support '
                            'PFS through Elliptic Curve Diffie-Hellman '
                            'Ephemeral (ECDHE) key exchange.'),
                    }
                    ats.append(findings)
                ct_tag = config.get('NSRequiresCertificateTransparency', False)
                if not ct_tag or ct_tag == 'NO':
                    findings = {
                        'issue': ('NSRequiresCertificateTransparency'
                                  ' set to NO for {}'.format(domain)),
                        'severity': 'warning',
                        'description': (
                            'Certificate Transparency (CT) is a protocol '
                            'that ATS can use to identify '
                            'mistakenly or maliciously '
                            'issued X.509 certificates. '
                            'Set the value for the '
                            'NSRequiresCertificateTransparency '
                            'key to YES to require that for a given domain, '
                            'server certificates are supported by valid, '
                            'signed CT timestamps from at least '
                            'two CT logs trusted by Apple. '
                            'This key is optional. The default value is NO.'),
                    }
                    ats.append(findings)
                elif ct_tag == 'YES':
                    findings = {
                        'issue': ('NSRequiresCertificateTransparency'
                                  ' set to YES for {}'.format(domain)),
                        'severity': 'secure',
                        'description': (
                            'Certificate Transparency (CT) is a protocol '
                            'that ATS can use to identify '
                            'mistakenly or maliciously '
                            'issued X.509 certificates. '
                            'Set the value for the '
                            'NSRequiresCertificateTransparency '
                            'key to YES to require that for a given domain, '
                            'server certificates are supported by valid, '
                            'signed CT timestamps from at least '
                            'two CT logs trusted by Apple. '
                            'This key is optional. The default value is NO.'),
                    }
                    ats.append(findings)

    return ats



# -*- coding: utf_8 -*-
"""iOS File Analysis."""

import shutil
import logging
from pathlib import Path

from django.utils.html import escape

from mobsf.StaticAnalyzer.views.ios.plist_analysis import (
    convert_bin_xml,
)
from mobsf.MobSF.utils import (
    append_scan_status,
)

logger = logging.getLogger(__name__)


def ios_list_files(md5_hash, src, mode):
    """List iOS files."""
    try:
        msg = 'iOS File Analysis and Normalization'
        logger.info(msg)
        append_scan_status(md5_hash, msg)
        # Multi function, Get Files, BIN Plist -> XML, normalize + to x
        filez = []
        certz = []
        sfiles = []
        full_paths = []
        database = []
        plist = []

        mode = 'ios' if mode == 'zip' else 'ipa'

        # Walk through the directory
        for file_path in Path(src).rglob('*'):
            if (file_path.is_file()
                    and not (file_path.name.endswith('.DS_Store')
                             or '__MACOSX' in str(file_path))):
                # Normalize '+' in file names
                if '+' in file_path.name:
                    normalized_path = file_path.with_name(
                        file_path.name.replace('+', 'x'))
                    shutil.move(file_path, normalized_path)
                    file_path = normalized_path

                # Append file details
                relative_path = file_path.relative_to(src)
                filez.append(str(relative_path))
                full_paths.append(str(file_path))

                ext = file_path.suffix.lower()

                # Categorize files by type
                if ext in {'.cer', '.pem', '.cert', '.crt', '.pub',
                           '.key', '.pfx', '.p12', '.der'}:
                    certz.append({
                        'file_path': escape(str(relative_path)),
                        'type': None,
                        'hash': None,
                    })
                elif ext in {'.db', '.sqlitedb', '.sqlite', '.sqlite3'}:
                    database.append({
                        'file_path': escape(str(relative_path)),
                        'type': mode,
                        'hash': md5_hash,
                    })
                elif ext in {'.plist', '.json'}:
                    if mode == 'ipa' and ext == '.plist':
                        convert_bin_xml(file_path.as_posix())
                    plist.append({
                        'file_path': escape(str(relative_path)),
                        'type': mode,
                        'hash': md5_hash,
                    })

        # Group special files
        if database:
            sfiles.append({
                'issue': 'SQLite Files',
                'files': database,
            })
        if plist:
            sfiles.append({
                'issue': 'Plist Files',
                'files': plist,
            })
        if certz:
            sfiles.append({
                'issue': 'Certificate/Key Files Hardcoded inside the App.',
                'files': certz,
            })

        return {
            'files_short': filez,
            'files_long': full_paths,
            'special_files': sfiles,
        }
    except Exception as exp:
        msg = 'iOS File Analysis'
        logger.exception(msg)
        append_scan_status(md5_hash, msg, repr(exp))



# -*- coding: utf_8 -*-
"""Module for apk analysis."""
import os
import re
import logging
from pathlib import Path

from mobsf.StaticAnalyzer.tools.androguard4 import (
    apk,
)
from mobsf.StaticAnalyzer.views.android import (
    aapt,
)
from mobsf.MobSF.utils import (
    append_scan_status,
)

logger = logging.getLogger(__name__)


def aapt_parse(app_dict):
    """Extract features from APK using aapt/aapt2."""
    checksum = app_dict['md5']
    app_dict['apk_features'] = {}
    app_dict['apk_strings'] = []
    try:
        msg = 'Extracting APK features using aapt/aapt2'
        logger.info(msg)
        append_scan_status(checksum, msg)
        aapt_obj = aapt.AndroidAAPT(app_dict['app_path'])
        app_dict['apk_features'] = aapt_obj.get_apk_features()
        if not app_dict.get('files'):
            app_dict['files'] = aapt_obj.get_apk_files()
        app_dict['apk_strings'] = aapt_obj.get_apk_strings()
    except FileNotFoundError:
        msg = 'aapt and aapt2 not found, skipping APK feature extraction'
        logger.warning(msg)
        append_scan_status(checksum, msg)
    except Exception as exp:
        msg = 'Failed to extract APK features using aapt/aapt2'
        logger.warning(msg)
        append_scan_status(checksum, msg, repr(exp))


def androguard_parse(app_dict):
    """Extract features from APK using androguard."""
    checksum = app_dict['md5']
    app_dict['androguard_apk'] = None
    app_dict['androguard_manifest_xml'] = None
    app_dict['androguard_apk_resources'] = None
    app_dict['androguard_apk_name'] = None
    app_dict['androguard_apk_icon'] = None
    try:
        msg = 'Parsing APK with androguard'
        logger.info(msg)
        append_scan_status(checksum, msg)
        a = apk.APK(app_dict['app_path'])
        if not a:
            msg = 'Failed to parse APK with androguard'
            logger.warning(msg)
            append_scan_status(checksum, msg)
            return
        app_dict['androguard_apk'] = a
        try:
            app_dict['androguard_apk_name'] = a.get_app_name()
        except Exception as exp:
            msg = 'Failed to get app name with androguard'
            logger.warning(msg)
            append_scan_status(checksum, msg, repr(exp))
        try:
            app_dict['androguard_apk_icon'] = a.get_app_icon(max_dpi=0xFFFE - 1)
        except Exception as exp:
            msg = 'Failed to get app icon with androguard'
            logger.warning(msg)
            append_scan_status(checksum, msg, repr(exp))
        try:
            xml = a.get_android_manifest_axml().get_xml()
            app_dict['androguard_manifest_xml'] = xml
        except Exception as exp:
            msg = 'Failed to parse AndroidManifest.xml with androguard'
            logger.warning(msg)
            append_scan_status(checksum, msg, repr(exp))
        try:
            app_dict['androguard_apk_resources'] = a.get_android_resources()
        except Exception as exp:
            msg = 'Failed to parse resources with androguard'
            logger.warning(msg)
            append_scan_status(checksum, msg, repr(exp))
    except Exception as exp:
        msg = 'Failed to parse APK with androguard'
        logger.error(msg)
        append_scan_status(checksum, msg, repr(exp))


def get_apk_name(app_dic):
    """Get app name."""
    real_name = ''
    base = Path(app_dic['app_dir'])

    # Check if it's an APK and try to retrieve the app name
    if app_dic.get('androguard_apk_name') or app_dic.get('apk_features'):
        app_name = (
            app_dic.get('androguard_apk_name')
            or app_dic.get('apk_features', {}).get('application_label')
        )
        if app_name:
            real_name = app_name
        else:
            # Fallback: Look for app_name in the values folder
            values_path = base / 'apktool_out' / 'res' / 'values'
            if values_path.exists():
                try:
                    real_name = get_app_name_from_values_folder(values_path.as_posix())
                except Exception:
                    logger.error('Failed to get app name from values folder')

    # Check if it's source code and try to retrieve the app name
    else:
        try:
            # Check paths for values folders
            paths_to_check = [
                base / 'app' / 'src' / 'main' / 'res' / 'values',
                base / 'res' / 'values',
            ]
            for path in paths_to_check:
                if path.exists():
                    real_name = get_app_name_from_values_folder(path.as_posix())
                    break
        except Exception:
            logger.error('Failed to get app name from source code')

    if not real_name:
        logger.warning('Cannot find app name')

    # Update the app dictionary
    app_dic['real_name'] = real_name


def get_app_name_from_values_folder(values_dir):
    """Get all the files in values folder and checks them for app_name."""
    files = [f for f in os.listdir(values_dir) if
             (os.path.isfile(os.path.join(values_dir, f)))
             and (f.endswith('.xml'))]
    for f in files:
        # Look through each file, searching for app_name.
        app_name = get_app_name_from_file(os.path.join(values_dir, f))
        if app_name:
            return app_name  # we found an app_name, lets return it.
    return ''  # Didn't find app_name, returning empty string.


def get_app_name_from_file(file_path):
    """Looks for app_name in specific file."""
    with open(file_path, 'r', encoding='utf-8') as f:
        data = f.read()

    app_name_match = re.search(
        r'<string name=\"app_name\">(.{0,300})</string>',
        data)

    if (not app_name_match) or (len(app_name_match.group()) <= 0):
        # Did not find app_name in current file.
        return ''

    # Found app_name!
    return app_name_match.group(app_name_match.lastindex)



"""Static Analysis Setup Windows.

Setup script for the Windows vm for usage with MobSF for
static analysis of Windows apps.
"""
import configparser
import logging
import os
import platform
import re
import shutil
import subprocess
import sys
from os.path import expanduser

from six.moves import input as sinput

try:
    import urllib.request as urlrequest
except ImportError:
    import urllib as urlrequest


logger = logging.getLogger(__name__)

# pylint: disable=C0325,W0603

# Only static URL, let's hope this never changes..
CONFIG_URL = (
    'https://raw.githubusercontent.com/MobSF/'
    'Mobile-Security-Framework-MobSF/master/'
    'mobsf/install/windows/config.txt'
)

# Static path to config file as a starting point
CONFIG_PATH = expanduser('~') + '\\MobSF\\Config\\'
CONFIG_FILE = 'config.txt'

# Static path to autostart
AUTOSTART = (
    expanduser('~') + '\\AppData\\Roaming\\Microsoft\\'
    'Windows\\Start Menu\\Programs\\Startup\\'
)

# Global var so we don't have to pass it every time..
CONFIG = ''


def windows_config_local(path):
    """Windows Configuration."""
    # Configure here if you are on windows
    # Path to lock-file (so setup is only run once)
    path_to_lock_file = os.path.join(path, 'setup_done.txt')
    if (os.path.isfile(path_to_lock_file) is False
            and platform.system() == 'Windows'
            and 'CI' not in os.environ):
        logger.info('Running first time setup for windows.')
        # Setup is to-be-executed
        install_locally(path)


def download_config():
    """Download initial config file."""
    # Create config path
    if not os.path.exists(CONFIG_PATH):
        os.makedirs(CONFIG_PATH)

    if os.path.exists(CONFIG_PATH + CONFIG_FILE):
        os.remove(CONFIG_PATH + CONFIG_FILE)

    # TODO(Give user time to modify config, but mayber after rewrite?)

    # Open File
    conf_file_local = open(CONFIG_PATH + CONFIG_FILE, 'wb')

    # Downloading File
    print('[*] Downloading config file..')
    conf_file = urlrequest.urlopen(CONFIG_URL)  # pylint: disable-msg=E1101

    # Save content
    print(('[*] Saving to File {}'.format(CONFIG_FILE)))

    # Write content to file
    conf_file_local.write(bytes(conf_file.read()))

    # Aaaand close
    conf_file_local.close()


def read_config():
    """Read the config file and write it to the global var."""
    print('[*] Reading config file..')

    global CONFIG
    CONFIG = configparser.ConfigParser()
    CONFIG.read(CONFIG_PATH + CONFIG_FILE)


def create_folders():
    """Create MobSF dirs."""
    print('[*] Creating other folders...')

    if not os.path.exists(CONFIG['MobSF']['downloads']):
        os.makedirs(CONFIG['MobSF']['downloads'])

    if not os.path.exists(CONFIG['MobSF']['tools']):
        os.makedirs(CONFIG['MobSF']['tools'])

    if not os.path.exists(CONFIG['MobSF']['samples']):
        os.makedirs(CONFIG['MobSF']['samples'])

    if not os.path.exists(CONFIG['MobSF']['key_path']):
        os.makedirs(CONFIG['MobSF']['key_path'])


def check_dependencies():
    """Check dependencies and install if necessary."""
    print('[*] Checking dependencies...')
    missing_deps = []

    try:
        import rsa  # noqa F401
        print('[+] rsa is installed.')
    except ImportError:  # pylint: disable-msg=C0103
        print('[!] rsa not installed!')
        missing_deps.append('rsa')

    if len(missing_deps) > 0:
        print('[!] Please install these missing dependencies:')
        print(missing_deps)
        sys.exit()
    else:
        print('[+] Everything good.')


def tools_nuget():
    """Download nuget."""
    # Get config params
    nuget_url = CONFIG['nuget']['url']
    mobsf_subdir_tools = CONFIG['MobSF']['tools']
    nuget_file_path = CONFIG['nuget']['file']

    # Open File
    nuget_file_local = open(os.path.join(
        mobsf_subdir_tools, nuget_file_path), 'wb')

    # Downloading File
    print('[*] Downloading nuget..')
    nuget_file = urlrequest.urlopen(nuget_url)  # pylint: disable-msg=E1101

    # Save content
    print(('[*] Saving to File {}'.format(nuget_file_path)))

    # Write content to file
    nuget_file_local.write(bytes(nuget_file.read()))

    # Aaaand close
    nuget_file_local.close()


def tools_binskim():
    """Download and extract binskim."""
    # Get dirs, urls etc.
    binskim_nuget = CONFIG['binskim']['nuget']
    mobsf_subdir_tools = CONFIG['MobSF']['tools']
    nuget = mobsf_subdir_tools + CONFIG['nuget']['file']

    print('[*] Downloading and installing Binskim...')
    # Execute nuget to get binkim
    output = subprocess.check_output(
        [
            nuget,
            'install', binskim_nuget, '-Pre',
            '-Version', '1.7.2',
            '-o', mobsf_subdir_tools])

    # Some code to determine the version on the fly so we don't have to fix the
    # config file on every new release of binskim..

    # Search for the version number
    folder = re.search(
        b'Microsoft\\.CodeAnalysis\\.BinSkim\\..{0,300}(\'|\") ', output)
    try:
        # Substring-Foo for removing b'X's in python3
        if sys.version_info.major == 3:
            folder = str(folder.group(0)[:-2])[2:-1]
        else:
            folder = folder.group(0)[:-2]
    except AttributeError:
        print('[!] Unable to parse folder from binskim nuget installation.')
        sys.exit()

    # Search for the exes
    binaries = _find_exe(mobsf_subdir_tools + folder, [])
    if len(binaries) != 2:
        print('[!] Found more than 2 exes for binskim, panic!')
        sys.exit()

    # Determinde which one is for which arch
    if 'x86' in binaries[0]:
        CONFIG['binskim']['file_x86'] = binaries[0]
        CONFIG['binskim']['file_x64'] = binaries[1]
    else:
        CONFIG['binskim']['file_x86'] = binaries[1]
        CONFIG['binskim']['file_x64'] = binaries[0]

    # Write to config
    with open(os.path.join(CONFIG_PATH, CONFIG_FILE), 'w') as configfile:
        CONFIG.write(configfile)  # pylint: disable-msg=E1101


def _find_exe(path, exe_list):
    """Return a list of all exes in path, recursive."""
    for filename in os.listdir(path):
        if os.path.isfile(os.path.join(path, filename)):
            if '.exe' in filename:
                exe_list.append(path + '\\' + filename)
        else:
            exe_list = _find_exe(path + '\\' + filename, exe_list)
    return exe_list


def tools_rpcclient():
    """Download and install rpc-server for MobSF."""
    rpc_url = CONFIG['rpc']['url']
    mobsf_subdir_tools = CONFIG['MobSF']['tools']
    rpc_file_path = CONFIG['rpc']['file']

    # Open File
    rpc_local_file = open(mobsf_subdir_tools + rpc_file_path, 'wb')

    # Downloading File
    print('[*] Downloading rpc_server..')
    rpc_file = urlrequest.urlopen(rpc_url)

    # Save content
    print(('[*] Saving to File {}'.format(rpc_file_path)))

    # Write content to file
    rpc_local_file.write(bytes(rpc_file.read()))

    # Aaaand close
    rpc_local_file.close()


def tools_binscope():
    """Download and install Binscope for MobSF."""
    mobsf_subdir_tools = CONFIG['MobSF']['tools']
    binscope_path = mobsf_subdir_tools + 'BinScope'

    # Download the right version for os
    if platform.machine().endswith('64'):
        binscope_url = CONFIG['binscope']['url_x64']
        binscope_installer_path = binscope_path + '\\BinScope_x64.msi'
    else:
        binscope_url = CONFIG['binscope']['url_x86']
        binscope_installer_path = binscope_path + '\\BinScope_x86.msi'

    if not os.path.exists(binscope_path):
        os.makedirs(binscope_path)

    binscope_installer_file = open(binscope_installer_path, 'wb')

    # Downloading File
    print('[*] Downloading BinScope..')
    binscope_installer = urlrequest.urlopen(binscope_url)

    # Save content
    print(('[*] Saving to File {}'.format(binscope_installer_path)))

    # Write content to file
    binscope_installer_file.write(bytes(binscope_installer.read()))

    # Aaaand close
    binscope_installer_file.close()
    # Execute the installer
    print(('[*] Installing BinScope to {}'.format(binscope_path)))
    cmd = ('msiexec INSTALLLOCATION='
           '"{}" /i "{}" /passive'.format(
               binscope_path,
               binscope_installer_path))
    os.system(cmd)

    CONFIG['binscope']['file'] = binscope_path + '\\Binscope.exe'

    # Write to config
    with open(os.path.join(CONFIG_PATH, CONFIG_FILE), 'w') as configfile:
        CONFIG.write(configfile)  # pylint: disable-msg=E1101


def generate_secret():
    """Generate rsa keys for authentication."""
    import rsa
    print('[*] Generating secret, please hang on.')
    # Generate keys, taken from
    # https://stuvel.eu/python-rsa-doc/usage.html#generating-keys
    (pubkey, privkey) = rsa.newkeys(2048)

    # Save private and pub key
    priv_key_file = open(CONFIG['MobSF']['priv_key'], 'w')
    priv_key_file.write(privkey.save_pkcs1().decode('utf-8'))
    priv_key_file.close()
    pub_key_file = open(CONFIG['MobSF']['pub_key'], 'w')
    pub_key_file.write(pubkey.save_pkcs1().decode('utf-8'))
    pub_key_file.close()
    config_path = os.path.join(
        expanduser('~'),
        '.MobSF',
        'config.py')
    print((
        '[!] Please move the private key file\n'
        '\t{}\n'
        '\tto MobSF to the path specified in {}\n'
        '\t(default: Mobile-Security-Framework-MobSF/'
        'mobsf/MobSF/windows_vm_priv_key.asc)'
        .format(CONFIG['MobSF']['priv_key'], config_path)
    ))
    sinput('Please press any key when done..')


def autostart():
    """Create the autostart binary and run it."""
    mobsf_subdir_tools = CONFIG['MobSF']['tools']
    rpc_file = CONFIG['rpc']['file']
    autostart_file = CONFIG['autostart']['file']
    batch_file = AUTOSTART + autostart_file

    print('[*] Creating autostart binary...')

    # Open file
    autostart_file = open(batch_file, 'wb')

    # Define bat-text
    text = """
    @echo off
    python "{}" %*
    pause""".format(mobsf_subdir_tools + rpc_file)
    autostart_file.write(bytes(text, 'utf8'))

    # Close handle
    autostart_file.close()

    print('[*] Done. Start the server.')

    # Execute. Beware the ' ' because of windows strange paths..
    os.system('"{}"'.format(batch_file))


def _place_lockfile(mobsf_home):
    path = os.path.join(mobsf_home, 'setup_done.txt')
    open(path, 'a').close()


def local_config():
    """Move local config and save paths."""
    # Set the CONFIG_PATH
    # Create path if it doesn't exist yet
    if not os.path.exists(CONFIG_PATH):
        os.makedirs(CONFIG_PATH)

    # Copy predefined config to MobSF folder
    shutil.copy(
        os.getcwd() + '\\mobsf\\install\\windows\\config.txt',
        os.path.join(CONFIG_PATH, CONFIG_FILE),
    )


def rewrite_local_config(mobsf_home):
    """For local installation some config-vars need to be rewritten."""
    CONFIG['MobSF']['subdir_tools'] = (
        mobsf_home
        + '\\mobsf\\StaticAnalyzer\\tools\\windows\\')
    CONFIG['MobSF']['dir'] = mobsf_home

    # Write to config
    with open(os.path.join(CONFIG_PATH, CONFIG_FILE), 'w') as configfile:
        CONFIG.write(configfile)  # pylint: disable-msg=E1101


def rewrite_config():
    """Rewrite the config to take the profile path as the base path."""
    # Take user path as base path
    CONFIG['MobSF']['dir'] = expanduser('~') + CONFIG['MobSF']['dir']

    # Rewrite config with new base path
    CONFIG['MobSF']['downloads'] = CONFIG['MobSF'][
        'dir'] + CONFIG['MobSF']['downloads']
    CONFIG['MobSF']['tools'] = CONFIG['MobSF'][
        'dir'] + CONFIG['MobSF']['tools']
    CONFIG['MobSF']['samples'] = CONFIG['MobSF'][
        'dir'] + CONFIG['MobSF']['samples']
    CONFIG['MobSF']['key_path'] = CONFIG['MobSF'][
        'dir'] + CONFIG['MobSF']['key_path']
    CONFIG['MobSF']['priv_key'] = CONFIG['MobSF'][
        'key_path'] + CONFIG['MobSF']['priv_key']
    CONFIG['MobSF']['pub_key'] = CONFIG['MobSF'][
        'key_path'] + CONFIG['MobSF']['pub_key']

    # Write to config
    with open(os.path.join(CONFIG_PATH, CONFIG_FILE), 'w') as configfile:
        CONFIG.write(configfile)  # pylint: disable-msg=E1101


def install_locally(mobsf_home):
    """Install the MobSF-Utils on the same system as MobSF."""
    local_config()
    read_config()
    rewrite_config()
    create_folders()
    tools_nuget()
    tools_binskim()
    tools_binscope()
    _place_lockfile(mobsf_home)


def _install_remote():
    """Install the MobSF-Utils on a Windows-VM for static analysis."""
    download_config()
    read_config()
    rewrite_config()
    create_folders()
    check_dependencies()
    tools_nuget()
    tools_binskim()
    tools_binscope()
    tools_rpcclient()
    generate_secret()
    autostart()


if __name__ == '__main__':
    # Gets directly run if setup.py is run on a remote machine
    _install_remote()



# -*- coding: utf_8 -*-
"""Android APK Downloader."""
import logging
from tempfile import gettempdir
from pathlib import Path
from zipfile import ZipFile

import requests

from bs4 import BeautifulSoup

from django.conf import settings

from mobsf.MobSF.views.scanning import (
    add_to_recent_scan,
    handle_uploaded_file,
)
from mobsf.MobSF.security import is_path_traversal
from mobsf.MobSF.utils import (
    is_internet_available,
    is_zip_magic,
    strict_package_check,
    upstream_proxy,
)


logger = logging.getLogger(__name__)


def fetch_html(url):
    """Get Result HTML."""
    headers = {
        'User-Agent': ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) '
                       'AppleWebKit/537.36 (KHTML, like Gecko) '
                       'Chrome/39.0.2171.95 Safari/537.36'),
        'Accept-Encoding': 'deflate, gzip'}
    try:
        proxies, verify = upstream_proxy('https')
        res = requests.get(url,
                           timeout=5,
                           headers=headers,
                           proxies=proxies,
                           verify=verify,
                           stream=True)
        if res.status_code == 200:
            return BeautifulSoup(res.text, features='lxml')
    except Exception:
        pass
    return None


def download_file(url, outfile):
    try:
        logger.info('Downloading APK...')
        proxies, verify = upstream_proxy('https')
        with requests.get(url,
                          timeout=5,
                          stream=True,
                          proxies=proxies,
                          verify=verify) as r:
            r.raise_for_status()
            with open(outfile, 'wb') as f:
                for chunk in r.iter_content(chunk_size=8192):
                    f.write(chunk)
        return outfile
    except Exception:
        pass
    return None


def get_scan_type(location):
    """Get APK type."""
    with ZipFile(location, 'r') as zf:
        for fil in zf.namelist():
            if fil.endswith('.apk'):
                return 'apks'
    return 'apk'


def add_apk(dwd_file, filename):
    """Add APK to MobSF."""
    with dwd_file.open('rb') as flip:
        if not is_zip_magic(flip):
            logger.warning('Downloaded file is not an APK/Split APK')
            return None
        md5 = handle_uploaded_file(flip, '.apk')
        apk = Path(settings.UPLD_DIR) / md5 / f'{md5}.apk'
        scan_type = get_scan_type(apk)
        data = {
            'analyzer': 'static_analyzer',
            'status': 'success',
            'hash': md5,
            'scan_type': scan_type,
            'file_name': filename,
        }
        add_to_recent_scan(data)
        return data
    return None


def find_apk_link(url, domain):
    """Find APK download link."""
    try:
        logger.info('Looking for download link form %s', domain)
        bsp = fetch_html(url)
        if not bsp:
            return None
        link = bsp.find('a', href=True, string='click here')
        if link:
            logger.info('Download link found from %s', domain)
            return link['href']
        logger.warning('Download link not found in %s', domain)
    except Exception:
        logger.warning('Failed to obtain download link from %s', domain)
    return None


def try_provider(package, provider, domain):
    """Try using a provider."""
    downloaded_file = None
    data = None
    apk_name = f'{package}.apk'
    temp_file = Path(gettempdir()) / apk_name
    link = find_apk_link(provider, domain)
    if link:
        downloaded_file = download_file(link, temp_file)
    if downloaded_file:
        data = add_apk(downloaded_file, apk_name)
    if data:
        return data
    return None


def apk_download(package):
    """Download APK."""
    downloaded_file = None
    data = None
    try:
        if not is_internet_available():
            logger.warning('Internet Not Available. Unable to download APK')
            return None
        if not strict_package_check(package) or is_path_traversal(package):
            return None
        logger.info('Attempting to download: %s', package)
        # APKTADA
        data = try_provider(
            package,
            f'{settings.APKTADA}{package}',
            'apktada.com')
        if data:
            return data
        # APKPURE
        data = try_provider(
            package,
            settings.APKPURE.format(package),
            'apkpure.com')
        if data:
            return data
        # APKPLZ
        data = try_provider(
            package,
            f'{settings.APKPLZ}{package}',
            'apkplz.net')
        if data:
            return data
        logger.warning('Unable to find download link for %s', package)
        return None
    except Exception:
        logger.exception('Failed to download the apk')
        return None
    finally:
        if downloaded_file:
            downloaded_file.unlink()



# -*- coding: utf_8 -*-
from google_play_scraper import app

from bs4 import BeautifulSoup

import requests

import logging

from django.conf import settings

from mobsf.MobSF.utils import (
    append_scan_status,
    upstream_proxy,
)

logger = logging.getLogger(__name__)


def get_app_details(app_dic, man_data):
    """Get App Details form PlayStore."""
    checksum = app_dic['md5']
    app_dic['playstore'] = {
        'error': True,
        'description': 'Failed to identify the package name',
    }
    try:
        # Check if Google Play Store is reachable
        # The library can cause timeout issue behind proxy
        try:
            proxies, verify = upstream_proxy('https')
            requests.get(settings.PLAYSTORE,
                         timeout=5,
                         proxies=proxies,
                         verify=verify)
        except Exception:
            logger.warning('Google Play Store is not reachable.'
                           ' Skipping Play Store lookup.')
            return

        if man_data.get('packagename'):
            package_id = man_data['packagename']
        elif app_dic.get('apk_features', {}).get('package'):
            package_id = app_dic['apk_features']['package']
        else:
            logger.warning('Package Name not found')
            return
        msg = f'Fetching Details from Play Store: {package_id}'
        logger.info(msg)
        append_scan_status(checksum, msg)
        det = app(package_id)
        det.pop('descriptionHTML', None)
        det.pop('comments', None)
        description = BeautifulSoup(det['description'], features='lxml')
        det['description'] = description.get_text()
        det['error'] = False
        if 'androidVersionText' not in det:
            det['androidVersionText'] = ''
    except Exception:
        det = app_search(checksum, package_id)
    app_dic['playstore'] = det


def app_search(checksum, app_id):
    """Get App Details from AppMonsta."""
    det = {'error': True}
    if not settings.APPMONSTA_API:
        return det
    msg = f'Fetching Details from AppMonsta: {app_id}'
    append_scan_status(checksum, msg)
    logger.info(msg)
    lookup_url = settings.APPMONSTA_URL
    req_url = '{}{}.json?country={}'.format(
        lookup_url, app_id, 'US')
    headers = {
        'User-Agent': ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) '
                       'AppleWebKit/537.36 (KHTML, like Gecko) '
                       'Chrome/39.0.2171.95 Safari/537.36'),
        'Accept-Encoding': 'deflate, gzip'}
    try:
        proxies, verify = upstream_proxy('https')
        req = requests.get(req_url,
                           timeout=5,
                           auth=(settings.APPMONSTA_API, 'X'),
                           headers=headers,
                           proxies=proxies,
                           verify=verify,
                           stream=True)
        resp = req.json()
        det['title'] = resp['app_name']
        det['score'] = resp.get('all_rating', '')
        det['installs'] = resp.get('downloads', '')
        det['price'] = resp.get('price', '')
        det['androidVersionText'] = resp.get('requires_os', '')
        det['genre'] = resp.get('genre', '')
        det['url'] = resp.get('store_url', '')
        det['developer'] = resp.get('publisher_name', '')
        det['developerId'] = resp.get('publisher_id', '')
        det['developerAddress'] = resp.get('publisher_address', '')
        det['developerWebsite'] = resp.get('publisher_url', '')
        det['developerEmail'] = resp.get('publisher_email', '')
        det['released'] = resp.get('release_date', '')
        det['privacyPolicy'] = resp.get('privacy_url', '')
        description = BeautifulSoup(resp.get('description', ''),
                                    features='lxml')
        det['description'] = description.get_text()
        det['error'] = False
        return det
    except Exception as exp:
        msg = 'Unable to get app details'
        append_scan_status(checksum, msg, repr(exp))
        logger.warning(msg)
        return det



# -*- coding: utf_8 -*-
"""Dynamic Analyzer Reporting."""
import logging
import os

from django.conf import settings
from django.shortcuts import render
from django.template.defaulttags import register

import mobsf.MalwareAnalyzer.views.Trackers as Trackers
from mobsf.DynamicAnalyzer.views.android.analysis import (
    generate_download,
    get_screenshots,
    run_analysis,
)
from mobsf.DynamicAnalyzer.views.android.operations import (
    get_package_name,
)
from mobsf.DynamicAnalyzer.views.android.tests_xposed import (
    droidmon_api_analysis,
)
from mobsf.DynamicAnalyzer.views.android.tests_frida import (
    apimon_analysis,
    dependency_analysis,
)
from mobsf.MobSF.utils import (
    is_file_exists,
    is_md5,
    key,
    print_n_send_error_response,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)

logger = logging.getLogger(__name__)
register.filter('key', key)


@login_required
def view_report(request, checksum, api=False):
    """Dynamic Analysis Report Generation."""
    logger.info('Dynamic Analysis Report Generation')
    try:
        droidmon = {}
        apimon = {}
        b64_strings = []
        if not is_md5(checksum):
            # We need this check since checksum is not validated
            # in REST API
            return print_n_send_error_response(
                request,
                'Invalid Hash',
                api)
        package = get_package_name(checksum)
        if not package:
            return print_n_send_error_response(
                request,
                'Invalid Parameters',
                api)
        app_dir = os.path.join(settings.UPLD_DIR, checksum + '/')
        download_dir = settings.DWD_DIR
        tools_dir = settings.TOOLS_DIR
        if not is_file_exists(os.path.join(app_dir, 'logcat.txt')):
            msg = ('Dynamic Analysis report is not available '
                   'for this app. Perform Dynamic Analysis '
                   'and generate the report.')
            return print_n_send_error_response(request, msg, api)
        fd_log = os.path.join(app_dir, 'mobsf_frida_out.txt')
        droidmon = droidmon_api_analysis(app_dir, package)
        apimon, b64_strings = apimon_analysis(app_dir)
        deps = dependency_analysis(package, app_dir)
        analysis_result = run_analysis(app_dir, checksum, package)
        domains = analysis_result['domains']
        trk = Trackers.Trackers(checksum, app_dir, tools_dir)
        trackers = trk.get_trackers_domains_or_deps(domains, deps)
        generate_download(app_dir, checksum, download_dir, package)
        images = get_screenshots(checksum, download_dir)
        context = {'hash': checksum,
                   'emails': analysis_result['emails'],
                   'urls': analysis_result['urls'],
                   'domains': domains,
                   'clipboard': analysis_result['clipboard'],
                   'xml': analysis_result['xml'],
                   'sqlite': analysis_result['sqlite'],
                   'others': analysis_result['other_files'],
                   'tls_tests': analysis_result['tls_tests'],
                   'screenshots': images['screenshots'],
                   'activity_tester': images['activities'],
                   'exported_activity_tester': images['exported_activities'],
                   'droidmon': droidmon,
                   'apimon': apimon,
                   'base64_strings': b64_strings,
                   'trackers': trackers,
                   'frida_logs': is_file_exists(fd_log),
                   'runtime_dependencies': list(deps),
                   'package': package,
                   'version': settings.MOBSF_VER,
                   'title': 'Dynamic Analysis'}
        template = 'dynamic_analysis/android/dynamic_report.html'
        if api:
            return context
        return render(request, template, context)
    except Exception as exp:
        logger.exception('Dynamic Analysis Report Generation')
        err = 'Error Generating Dynamic Analysis Report. ' + str(exp)
        return print_n_send_error_response(request, err, api)
